==> tombi-0.1.35.tar.gz: tombi-0.1.35/crates/formatter/src/format/table.rs <==
use crate::Format;
use itertools::Itertools;
use std::fmt::Write;

impl Format for ast::Table {
    fn fmt(&self, f: &mut crate::Formatter) -> Result<(), std::fmt::Error> {
        let header = self.header().unwrap();

        self.header_leading_comments().collect::<Vec<_>>().fmt(f)?;

        write!(f, "[{header}]")?;

        if let Some(comment) = self.header_tailing_comment() {
            comment.fmt(f)?;
        }

        let key_values = self.key_values().collect_vec();

        if key_values.is_empty() {
            let dangling_comments = self.dangling_comments();

            if !dangling_comments.is_empty() {
                write!(f, "{}", f.line_ending())?;
                dangling_comments.fmt(f)?;
            }

            return Ok(());
        } else {
            write!(f, "{}", f.line_ending())?;

            self.begin_dangling_comments().fmt(f)?;

            for (i, key_value) in key_values.into_iter().enumerate() {
                if i != 0 {
                    write!(f, "{}", f.line_ending())?;
                }
                key_value.fmt(f)?;
            }

            self.end_dangling_comments().fmt(f)?;
        }

        Ok(())
    }
}

#[cfg(test)]
mod tests {
    use crate::test_format;

    test_format! {
        #[test]
        fn table_only_header(
            r#"[package]"#
        ) -> Ok(source);
    }

    test_format! {
        #[test]
        fn table_only_header_with_basic_string_key(
            r#"[dependencies."unicase"]"#
        ) -> Ok(source);
    }

    test_format! {
        #[test]
        fn table_only_header_nexted_keys(
            r#"[dependencies.unicase]"#
        ) -> Ok(source);
    }

    test_format! {
        #[test]
        fn table(
            r#"
            [package]
            name = "toml-rs"
            version = "0.4.0"
            "#
        ) -> Ok(source);
    }

    test_format! {
        #[test]
        fn table_with_full_comment(
            r#"
            # header leading comment1
            # header leading comment2
            [header]  # header tailing comment
            # table begin dangling comment group 1-1
            # table begin dangling comment group 1-2

            # table begin dangling comment group 2-1
            # table begin dangling comment group 2-2
            # table begin dangling comment group 2-3

            # table begin dangling comment group 3-1

            # key value leading comment1
            # key value leading comment2
            key = "value"  # key tailing comment
            "#
        ) -> Ok(source);
    }

    test_format! {
        #[test]
        fn table_begin_dangling_comment1(
            r#"
            [header]
            # table begin dangling comment group 1-1
            # table begin dangling comment group 1-2

            # table begin dangling comment group 2-1
            # table begin dangling comment group 2-2
            # table begin dangling comment group 2-3

            # table begin dangling comment group 3-1

            # key value leading comment1
            # key value leading comment2
            key = "value"  # key tailing comment
            "#
        ) -> Ok(source);
    }

    test_format! {
        #[test]
        fn table_end_dangling_comment1(
            r#"
            [header]
            key = "value"  # key tailing comment
            # key value end dangling comment 1-1
            # key value end dangling comment 1-2
            "#
        ) -> Ok(source);
    }

    test_format! {
        #[test]
        fn table_end_dangling_comment2(
            r#"
            [header]
            key = "value"  # key tailing comment
            # key value end dangling comment 1-1
            # key value end dangling comment 1-2

            # key value end dangling comment 2-1
            # key value end dangling comment 2-2
            # key value end dangling comment 2-3

            # key value end dangling comment 3-1
            "#
        ) -> Ok(source);
    }

    test_format! {
        #[test]
        fn table_end_dangling_comment3(
            r#"
            [header]
            key = "value"  # key tailing comment

            # key value end dangling comment1
            # key value end dangling comment2
            "#
        ) -> Ok(source);
    }
}

==> tombi-0.2.20.tar.gz: tombi-0.2.20/crates/formatter/src/format/table.rs <==
use std::fmt::Write;

use itertools::Itertools;

use crate::Format;

impl Format for ast::Table {
    fn format(&self, f: &mut crate::Formatter) -> Result<(), std::fmt::Error> {
        let header = self.header().unwrap();

        self.header_leading_comments()
            .collect::<Vec<_>>()
            .format(f)?;

        write!(f, "[{header}]")?;

        if let Some(comment) = self.header_tailing_comment() {
            comment.format(f)?;
        }

        let key_values = self.key_values().collect_vec();

        if key_values.is_empty() {
            let dangling_comments = self.dangling_comments();

            if !dangling_comments.is_empty() {
                write!(f, "{}", f.line_ending())?;
                dangling_comments.format(f)?;
            }

            return Ok(());
        } else {
            write!(f, "{}", f.line_ending())?;

            self.begin_dangling_comments().format(f)?;

            for (i, key_value) in key_values.into_iter().enumerate() {
                if i != 0 {
                    write!(f, "{}", f.line_ending())?;
                }
                key_value.format(f)?;
            }

            self.end_dangling_comments().format(f)?;
        }

        Ok(())
    }
}

#[cfg(test)]
mod tests {
    use crate::test_format;

    test_format! {
        #[test]
        fn table_only_header(
            r#"[package]"#
        ) -> Ok(source);
    }

    test_format! {
        #[test]
        fn table_only_header_with_basic_string_key(
            r#"[dependencies."unicase"]"#
        ) -> Ok(source);
    }

    test_format! {
        #[test]
        fn table_only_header_nexted_keys(
            r#"[dependencies.unicase]"#
        ) -> Ok(source);
    }

    test_format! {
        #[test]
        fn table(
            r#"
            [package]
            name = "toml-rs"
            version = "0.4.0"
            "#
        ) -> Ok(source);
    }

    test_format! {
        #[test]
        fn table_with_full_comment(
            r#"
            # header leading comment1
            # header leading comment2
            [header]  # header tailing comment
            # table begin dangling comment group 1-1
            # table begin dangling comment group 1-2

            # table begin dangling comment group 2-1
            # table begin dangling comment group 2-2
            # table begin dangling comment group 2-3

            # table begin dangling comment group 3-1

            # key value leading comment1
            # key value leading comment2
            key = "value"  # key tailing comment
            "#
        ) -> Ok(source);
    }

    test_format! {
        #[test]
        fn table_begin_dangling_comment1(
            r#"
            [header]
            # table begin dangling comment group 1-1
            # table begin dangling comment group 1-2

            # table begin dangling comment group 2-1
            # table begin dangling comment group 2-2
            # table begin dangling comment group 2-3

            # table begin dangling comment group 3-1

            # key value leading comment1
            # key value leading comment2
            key = "value"  # key tailing comment
            "#
        ) -> Ok(source);
    }

    test_format! {
        #[test]
        fn table_end_dangling_comment1(
            r#"
            [header]
            key = "value"  # key tailing comment
            # key value end dangling comment 1-1
            # key value end dangling comment 1-2
            "#
        ) -> Ok(source);
    }

    test_format! {
        #[test]
        fn table_end_dangling_comment2(
            r#"
            [header]
            key = "value"  # key tailing comment
            # key value end dangling comment 1-1
            # key value end dangling comment 1-2

            # key value end dangling comment 2-1
            # key value end dangling comment 2-2
            # key value end dangling comment 2-3

            # key value end dangling comment 3-1
            "#
        ) -> Ok(source);
    }

    test_format! {
        #[test]
        fn table_end_dangling_comment3(
            r#"
            [header]
            key = "value"  # key tailing comment

            # key value end dangling comment1
            # key value end dangling comment2
            "#
        ) -> Ok(source);
    }
}
